=== modified file 'NEWS'
--- NEWS	2010-10-11 23:21:22 +0000
+++ NEWS	2010-10-11 23:54:01 +0000
@@ -70,6 +70,10 @@
 * Add ``annotate`` revision specifier, which selects the revision that
   introduced a specified line of a file.  (Aaron Bentley)
 
+* Add ``-Dmem_dump`` debug flag, which uses meliae to dump memory to 
+  a file upon an out of memory error.
+  (Karl Bielefeldt, #551391)
+
 * ``bzr status`` now displays a summary of existing shelves after
   the other status information. This is done using a ``post_status``
   hook.

=== modified file 'bzrlib/help_topics/en/debug-flags.txt'
--- bzrlib/help_topics/en/debug-flags.txt	2010-01-05 04:30:07 +0000
+++ bzrlib/help_topics/en/debug-flags.txt	2010-10-06 03:20:28 +0000
@@ -25,6 +25,7 @@
 -Dknit            Trace knit operations.
 -Dlock            Trace when lockdir locks are taken or released.
 -Dprogress        Trace progress bar operations.
+-Dmem_dump        Dump memory to a file upon an out of memory error.
 -Dmerge           Emit information for debugging merges.
 -Dno_apport       Don't use apport to report crashes.
 -Dno_activity 	  Don't show transport activity indicator in progress bar.

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2010-09-27 19:31:45 +0000
+++ bzrlib/tests/features.py	2010-10-11 09:53:16 +0000
@@ -38,12 +38,14 @@
 
 
 not_running_as_root = _NotRunningAsRoot()
+
 apport = tests.ModuleAvailableFeature('apport')
+meliae = tests.ModuleAvailableFeature('meliae')
 paramiko = tests.ModuleAvailableFeature('paramiko')
 pycurl = tests.ModuleAvailableFeature('pycurl')
 pywintypes = tests.ModuleAvailableFeature('pywintypes')
+sphinx = tests.ModuleAvailableFeature('sphinx')
 subunit = tests.ModuleAvailableFeature('subunit')
-sphinx = tests.ModuleAvailableFeature('sphinx')
 
 
 class _BackslashDirSeparatorFeature(tests.Feature):

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/test_trace.py	2010-10-11 09:53:16 +0000
@@ -26,6 +26,7 @@
 import tempfile
 
 from bzrlib import (
+    debug,
     errors,
     trace,
     )
@@ -81,7 +82,18 @@
             pass
         msg = _format_exception()
         self.assertEquals(msg,
-            "bzr: out of memory\n")
+            "bzr: out of memory\nUse -Dmem_dump to dump memory to a file.\n")
+
+    def test_format_mem_dump(self):
+        self.requireFeature(features.meliae)
+        debug.debug_flags.add('mem_dump')
+        try:
+            raise MemoryError()
+        except MemoryError:
+            pass
+        msg = _format_exception()
+        self.assertStartsWith(msg,
+            "bzr: out of memory\nMemory dumped to ")
 
     def test_format_os_error(self):
         try:

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2010-06-28 02:41:22 +0000
+++ bzrlib/trace.py	2010-10-05 23:30:18 +0000
@@ -59,6 +59,7 @@
 import os
 import sys
 import time
+import tempfile
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -469,6 +470,25 @@
                     note(line)
                     break
 
+def _dump_memory_usage(err_file):
+    try:
+        try:
+            fd, name = tempfile.mkstemp(prefix="bzr_memdump", suffix=".json")
+            dump_file = os.fdopen(fd, 'w')
+            from meliae import scanner
+            scanner.dump_gc_objects(dump_file)
+            err_file.write("Memory dumped to %s\n" % name)
+        except ImportError:
+            err_file.write("Dumping memory requires meliae module.\n")
+            log_exception_quietly()
+        except:
+            err_file.write("Exception while dumping memory.\n")
+            log_exception_quietly()
+    finally:
+        if dump_file is not None:
+            dump_file.close()
+        elif fd is not None:
+            os.close(fd)
 
 def report_exception(exc_info, err_file):
     """Report an exception to err_file (typically stderr) and to .bzr.log.
@@ -492,6 +512,10 @@
         return errors.EXIT_ERROR
     elif isinstance(exc_object, MemoryError):
         err_file.write("bzr: out of memory\n")
+        if 'mem_dump' in debug.debug_flags:
+            _dump_memory_usage(err_file)
+        else:
+            err_file.write("Use -Dmem_dump to dump memory to a file.\n")
         return errors.EXIT_ERROR
     elif isinstance(exc_object, ImportError) \
         and str(exc_object).startswith("No module named "):

